"""
Emojis for Python
~~~~~~~~~~~~~~~~~~~

A basic emoji manipulator for Python.

:copyright: (c) 2025 Omkaar
:license: MIT, see LICENSE for more details.
"""


__title__ = "emojis.py"
__author__ = "Ombucha"
__license__ = "MIT"
__copyright__ = "Copyright 2025 Omkaar"
__version__ = "1.3.0"


from .emojis import *
from .utils import *
